using System;
using System.Text;
using System.Net.Sockets;
using System.Net;
using System.Threading;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using SIPparser;
using GeoPos;
using MAC;
using GPS;

namespace ClientProxy
{
    public delegate void MessageEventHandler(String text);

    class SOSUserAgent
    {
        private string loc_source = "GeoPos"; //Location source
        private string username = "test3:TT";
        private string password = "Test856";
        private string location = "NOLOCATION";
        private string mac = "";
        GeoPos.GeoPos gp = null;

        SIPparser.SIPparser sipParser;
        SOSParser sosParser;
        ClientProxy form;
        private GpsPosition pos;
        string remotehost, remoteport;
        private MessageEventHandler messageHandlerDebug;

        #region Network related variables

        private Socket localSocket, remoteSocket;
        private EndPoint localEP, remoteClientEP, remoteServerEP, remoteEP;

        private static EndPoint localhost = (EndPoint)new IPEndPoint(IPAddress.Parse("127.0.0.1"), 5060);

        private const int incomingPort = 7000;
        private const int sipPort = 5060;

        private byte[] outgoingBuffer = new byte[1200];
        private byte[] incomingBuffer = new byte[1200];
        private byte[] globalBuffer;

        #endregion

        public SOSUserAgent()
        {
            //Empty constructor
        }

        //Constructor for agent that handles communcation between sip client and proxy
        //Emergency calls are handled and messages are supplemented with location data
        public SOSUserAgent(ClientProxy form, string remotehost, string remoteport, string localport)
        {
            //Setting remote host & port
            this.remotehost = remotehost;
            this.remoteport = remoteport;

            //Creating new SIP & SOS parser
            sipParser = new SIPparser.SIPparser(remotehost, remoteport, localport);
            sosParser = new SOSParser();

            //Create form to handle
            this.form = form;

            //Create message event handlers
            messageHandlerDebug = new MessageEventHandler(form.writeMessageDebug);

            try
            {
                //Local endpoints and its corresponding socket
                localEP = (EndPoint)new IPEndPoint(IPAddress.Any, incomingPort);
                remoteEP = (EndPoint)new IPEndPoint(IPAddress.Any, sipPort);

                localSocket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
                remoteSocket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
                
                localSocket.Bind(localEP);

                //DEBUG
                form.tb_debug.Text = String.Concat(GetType(), ": Bound to socket ", localEP.ToString());
                //form.Invoke(messageHandlerDebug, new object[] { GetType() + ": Bound to socket " + localEP.ToString() });
            }
            catch
            {
                throw;
            }
        }

        public SOSParser SOSParser
        {
            get
            {
                throw new System.NotImplementedException();
            }
            set
            {
            }
        }

        public MessageEventHandler MessageEventHandler
        {
            get
            {
                throw new System.NotImplementedException();
            }
            set
            {
            }
        }

        private void beginListen()
        {
            try
            {
                globalBuffer = new byte[1200];
                localSocket.BeginReceiveFrom(globalBuffer, 0, globalBuffer.Length, SocketFlags.None, ref remoteEP, new AsyncCallback(endListen), null);
            }
            catch
            {
                throw;
            }
        }

        private void endListen(IAsyncResult iar)
        {
            try
            {
                int bytesReceived = localSocket.EndReceiveFrom(iar, ref remoteEP);

                byte[] buffer = new byte[bytesReceived - 1];
                Array.Copy(globalBuffer, buffer, bytesReceived - 1);

                //DEBUG
                //form.Invoke(messageHandlerDebug, new object[] { "[Outgoing size]=" + buffer.Length });

                //forward to proxy if received from client
                if (remoteEP.Equals(localhost))
                {
                    //Parse outgoing message
                    sipParser.changeOutgoing(ref buffer);

                    //Sjekk om det er en INVITE melding
                    string[] stringA = ASCIIEncoding.ASCII.GetString(buffer, 0, buffer.Length).Split(Convert.ToChar("\n"));

                    if (stringA[0].StartsWith("INVITE"))
                    {
                        //Check if it is a SOS message
                        if (sosParser.isSOS(buffer))
                        {
                            string sosMessage = "";

                            //Location
                            string loc_coord = "";
                            string loc_format = "";

                            //Location source/service
                            string loc_service = String.Concat("a=sossource:", loc_source, "\r\n");
    
                            //Retrieve MAC address (local and NAP)
                            MAC.MAC m = new MAC.MAC();
                            mac = m.getAddress("LMAC");
                            string loc_mac_local = String.Concat("a=sosmac:", mac, "\r\n");
                            

                            //Time check                     
                           
                            DateTime timeBeforeLocation; 
                            DateTime timeAfterLocation;
                            
                            //Add location to SIP INVITE message
                            if(loc_source == "GeoPos")
                            {
                                string loc_mac_nap = String.Concat("a=sosnapmac:", m.getAddress("NMAC"), "\r\n");
                                
                                loc_format = String.Concat("a=sosformat:", "UTM", "\r\n");

                                gp = new GeoPos.GeoPos();
                                timeBeforeLocation = DateTime.Now;
                                
                                DateTime check = DateTime.Now;
                                TimeSpan limit = new TimeSpan();
                                limit = check.Subtract(timeBeforeLocation);

                                Thread t = new Thread(new ThreadStart(timerGeoPos));
                                t.Start();

                                //Timer for GeoPos
                                while( limit.TotalSeconds < 5 )
                                {
                                    check = DateTime.Now;
                                    limit = check.Subtract(timeBeforeLocation);
                                }
                                t.Abort();

                                if (location == "NOLOCATION")
                                {
                                    //location = "569477.0 7034368.0";
                                }
                                //location = gp.getLocationFor(username, password, "MAC", mac.ToLower(), loc_source);
                                timeAfterLocation = DateTime.Now;                                
                                
                                //location = "569477.0 7034368.0";
                                loc_coord = String.Concat("a=soscoord:", location, "\r\n");

                                //SOS message for GeoPos
                                sosMessage = String.Concat(loc_coord, loc_format, loc_service, loc_mac_local, loc_mac_nap);
                            }
                            else if (loc_source == "GPS")
                            {
                                loc_format = String.Concat("a=sosformat:", "WGS84", "\r\n");

                                //Henter GPS koordinater
                                //tb_main.Text = "Searching for GPS..";
                                form.Invoke(messageHandlerDebug, new object[] { "Searching for GPS..." });
                                timeBeforeLocation = DateTime.Now;
                                GPS.GPS gps = new GPS.GPS();        
                                gps.Open();
                                gps.LocationChanged += new LocationChangedEventHandler(gps_LocationChanged);
                                gps.DeviceStateChanged += new DeviceStateChangedEventHandler(gps_DeviceStateChanged);
                                GPS.GpsPosition gps_pos = gps.GetPosition();
                                timeAfterLocation = DateTime.Now;

                                loc_coord = String.Concat("a=soscoord:", gps_pos.Latitude.ToString() + " " + gps_pos.Longitude.ToString() , "\r\n");

                                //SOS message for GPS
                                sosMessage = String.Concat(loc_coord, loc_format, loc_service, loc_mac_local);
                            }
                            else
                            {
                                //Unknow location source
                                timeBeforeLocation = DateTime.Now;
                                loc_format = String.Concat("a=sosformat:", "UNKNOWN", "\r\n");
                                timeAfterLocation = DateTime.Now;
                                //SOS message for GeoPos
                                sosMessage = loc_format;
                            }

                            TimeSpan ts = new TimeSpan();
                            ts = timeAfterLocation.Subtract(timeBeforeLocation);

                            string sosdelay = String.Concat("a=sosdelay:", ts.ToString(), "\r\n");

                            //CREATE SOS MESSAGE
                            sosMessage = String.Concat(ASCIIEncoding.ASCII.GetString(buffer, 0, buffer.Length), sosMessage, sosdelay);
                            buffer = ASCIIEncoding.ASCII.GetBytes(sosMessage);
                        }
                    }

                    //DEBUG: Write out to form
                    //form.Invoke(messageHandlerOutParsed, new object[] { ASCIIEncoding.ASCII.GetString(buffer, 0, buffer.Length) });

                    //Send parsed outgoing message to server
                    beginForwardToServer(buffer);
                }
                //forward to client
                else
                {
                    //Parse incoming message
                    sipParser.changeIncoming(ref buffer);

                    //DEBUG: Write out to form
                    //form.Invoke(messageHandlerInParsed, new object[] { ASCIIEncoding.ASCII.GetString(buffer, 0, buffer.Length) });

                    //Send parsed incoming message to client
                    beginForwardToClient(buffer);
                }
                beginListen();
            }
            catch
            {
                throw;
            }
        }

        private void beginForwardToServer(byte[] buffer)
        {
            try
            {
                localSocket.BeginSendTo(buffer, 0, buffer.Length, SocketFlags.None, remoteServerEP, new AsyncCallback(endForwardToServer), remoteServerEP);
            }
            catch
            {
                throw;
            }
        }

        private void endForwardToServer(IAsyncResult iar)
        {
            try
            {
                int bytesSent = localSocket.EndSendTo(iar);
            }
            catch
            {
                throw;
            }
        }

        private void beginForwardToClient(byte[] buffer)
        {
            try
            {
                localSocket.BeginSendTo(buffer, 0, buffer.Length, SocketFlags.None, remoteClientEP, new AsyncCallback(endForwardToClient), remoteClientEP);
            }
            catch
            {
                throw;
            }
        }

        private void endForwardToClient(IAsyncResult iar)
        {
            try
            {
                int bytesSent = localSocket.EndSendTo(iar);
            }
            catch
            {
                throw;
            }
        }

        //Function ends a pending asynchonous send to proxy endpoint
        private void onSendToProxy(IAsyncResult iar)
        {
            if (iar.AsyncState.Equals(remoteServerEP))
            {
                try
                {
                    iar.AsyncWaitHandle.WaitOne();
                    int bytesSent = localSocket.EndSendTo(iar);
                }
                catch
                {
                    throw;
                }
            }
        }

        //Function sends data asynchronously to client endpoint
        private void beginSendToClient(byte[] buffer)
        {
            try
            {
                IAsyncResult iar = localSocket.BeginSendTo(buffer, 0, buffer.Length, SocketFlags.None, remoteClientEP, new AsyncCallback(onSendToClient), remoteClientEP);
                iar.AsyncWaitHandle.WaitOne();
            }
            catch
            {
                throw;
            }
        }

        //Function ends a pending asynchonous send to proxy endpoint
        private void onSendToClient(IAsyncResult iar)
        {
            try
            {
                if (iar.AsyncState.Equals(remoteClientEP))
                {
                    int bytesSent = localSocket.EndSendTo(iar);
                }
            }
            catch
            {
                throw;
            }
        }

        //Function that begins transmission between client and server
        public void start(string ls)
        {
            loc_source = ls;
            try
            {
                //Create remote endpoints for both client and server connection
                remoteClientEP = (EndPoint)new IPEndPoint(IPAddress.Parse("127.0.0.1"), 5060);
                remoteServerEP = (EndPoint)new IPEndPoint(IPAddress.Parse("129.241.200.44"), 5060);

                //Begin asynchronously udp transmission between client and proxy
                beginListen();

            }
            catch
            {
                throw;
            }
        }

        //Function closes active connections
        public void dispose()
        {
            //gps.Close();
            localSocket.Close();
            //remoteSocket.Close();
        }
        private void gps_LocationChanged(object sender, LocationChangedEventArgs args)
        {
            pos = args.Position;
            //latitude = pos.LongitudeRaw;
            //longitude = pos.LatitudeRaw;
        }
        private void gps_DeviceStateChanged(object sender, DeviceStateChangedEventArgs args)
        {
        }

        public void timerGeoPos()
        {
            location = gp.getLocationFor(username, password, "MAC", mac.ToLower(), loc_source);
        }    
    }
}